import asyncio
import logging
from typing import AsyncGenerator
from aiconsole.aic_types import ExecutionModeContext
from aiconsole.execution_modes.setup_openai_coding_llm import coding_llm
from aiconsole.gpt.create_full_prompt_from_sections import create_full_prompt_from_sections
from aiconsole.utils.convert_messages import convert_messages
from aiconsole.utils.get_user_info_string import get_user_info_string
import litellm

import logging

_log = logging.getLogger(__name__)

litellm.set_verbose = False
max_budget = None


async def execution_mode_interpreter(
    context: ExecutionModeContext,
) -> AsyncGenerator[str, None]:
    global llm

    # These are set directly on LiteLLM
    if max_budget:
        litellm.max_budget = max_budget

    system_message = create_full_prompt_from_sections(
        intro=context.agent.system,
        sections=context.relevant_materials,
        outro='\n\n---\n\n'
    )

    system_message += "\n\n" + get_user_info_string()

    _log.debug(f"System message:\n{system_message}")

    async for chunk in coding_llm(
            messages=[message for message in convert_messages(context.messages)],
            system_message=system_message,
            gpt_mode=context.agent.gpt_mode,
            min_tokens=250,
            preffered_tokens=2000):
        try:
            _log.debug(chunk)
            yield chunk
        except asyncio.CancelledError:
            _log.warning("Cancelled execution_mode_interpreter")
            break
