from typing import List, Optional
from pydantic import BaseModel
from aiconsole.gpt.types import GPTResponse, GPTRole
import logging

_log = logging.getLogger(__name__)


class GPTPartialFumctionCall(BaseModel):
    name: str = ""
    arguments: str = ""


class GPTPartialMessage(BaseModel):
    role: Optional[GPTRole] = None
    content: Optional[str] = None
    function_call: Optional[GPTPartialFumctionCall] = None
    name: Optional[str] = None


class GPTPartialChoice(BaseModel):
    index: int = 0
    message: GPTPartialMessage = GPTPartialMessage()
    role: str = ""
    finnish_reason: str = ""


class GPTPartialResponse(BaseModel):
    id: str = ""
    object: str = ""
    created: int = 0
    model: str = ""
    choices: List[GPTPartialChoice] = []

    def to_final_response(self):
        _log.debug(f"Partial response: {self.model_dump()}")
        return GPTResponse(**self.model_dump())

    def apply_chunk(self, chunk: dict):

        if "id" in chunk:
            self.id = chunk["id"]

        if "error" in chunk:
            raise ValueError(chunk["error"])
        
        if "object" in chunk:
            self.object = chunk["object"]

        if "created" in chunk:
            self.created = chunk["created"]

        if "model" in chunk:
            self.model = chunk["model"]

        if "choices" in chunk:
            chunk_choices = chunk["choices"]

            for chunk_choice in chunk_choices:
                index = None

                if "index" in chunk_choice:
                    index = int(chunk_choice["index"])

                if index is None:
                    raise ValueError("index not found")

                if index >= len(self.choices):
                    self.choices.append(GPTPartialChoice())

                choice = self.choices[index]
                choice.index = index
                
                if "finish_reason" in chunk_choice:
                    choice.finnish_reason = chunk_choice["finish_reason"]

                if "role" in chunk_choice:
                    choice.role = chunk_choice["role"]

                message = choice.message

                if "delta" in chunk_choice:
                    chunk_delta = chunk_choice["delta"]

                    if "name" in chunk_delta:
                        choice.message.name = chunk_delta["name"]

                    if "content" in chunk_delta:
                        message.content = chunk_delta["content"]

                    if "role" in chunk_delta:
                        message.role = chunk_delta["role"]

                    if "function_call" in chunk_delta:
                        chunk_function_call = chunk_delta["function_call"]

                        if not message.function_call:
                            message.function_call = GPTPartialFumctionCall()

                        if "name" in chunk_function_call:
                            message.function_call.name = chunk_function_call["name"]
                        
                        if "arguments" in chunk_function_call:
                            message.function_call.arguments += chunk_delta["function_call"]["arguments"]
                    else:
                        if "content" in chunk_delta:
                            if not choice.message.content:
                                choice.message.content = ""
                            
                            choice.message.content += chunk_delta["content"]

